/** \file
 * Header for the IAtlAtlObjectDefinition interface
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Richard Young
 * \version 1.0
 * \date 26/04/2004
 */

#ifndef IAtlAtlObjectDefinition_h
#define IAtlAtlObjectDefinition_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif
#ifndef	CAtlMatrix_h
	#include "CAtlMatrix.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// Forward declaration
class IAtl3DPlant;
class IAtlObjectLight;
class IAtlAnimatedObject;

/// List of properties
typedef TAtlObjectMgr<class CAtlProperty>			AtlPropertyMgr;

/// List of lights groups.
typedef TAtlObjectMgr<class IAtlLightsGroup>		AtlLightsGroupMgr;

/// List of points
typedef TAtlObjectMgr<class CAtlVector>			AtlPointMgr;

/// List of polygons
typedef TAtlObjectMgr<class CAtlPolygon>		AtlPolygonMgr;


/// Interface for atl objet definition
/** \interface IAtlAtlObjectDefinition 
 *	Class that contains the definitionn of an atl object
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Richard Young
 * \version 1.0
 * \date 27/04/2004
 */
class IAtlAtlObjectDefinition
{
public:

	/*! Enumeration of possible placement
		\enum eObjectPlacement
		\brief definition of possibles placement
	*/
	enum	eObjectPlacement
	{
		kPerpendicular,			//! perpendicular with the support
		kVertical				//! allways vertical
	};
	

	/*! Enumeration of possible model origin
		\enum eModelOrigin
		\brief definition of possibles model origin placement
	*/
	enum	eModelOrigin
	{
		kUnitialized,
		kCenterXYBottom,
		kCenterXYTop,
		kCenterXY,
		kCenterCameraTarget
	};

	// Values for the "geometry level" attribute of internal component.
	enum EGeometryLevel { 
		kGeometryLevelFromModeler = 0,	// i.e from an exporter
		kGeometryLevelStatic = 1,		// from Artlantis. 
		kGeometryLevelEditable = 2	// from Artlantis and editable.
	 }; // 

	/*!
	\interface	IAtlAtlObjectSetup
	\date		04 06 2007
	\author		Abvent R&D Copyright (C) 2007
	\author		Guillaume Boulet-Gercourt
	\brief		interface for access to atl object setup
	A setup is a configuration for the object properties
	*/
	class	IAtlAtlObjectSetup
	{
	public:
		/// Return the localized names of this object setup
		/** For consulting */
		virtual const IAtlString&		getName() const = 0;

		/// Return the localized names of this object setup
		/** For editing */
		virtual IAtlString&				getName() = 0;

		/// Return the id
		virtual tUUID					getId() const = 0;

		/// Set the id
		virtual void					setId(tUUID inId) = 0;
	protected:
		/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
		virtual ~IAtlAtlObjectSetup() {}
	};

	/// List of object setups availiable
	typedef TAtlObjectMgr<class IAtlAtlObjectSetup>	AtlSetupsMgr;


	/// Put an XML representation of the object definition inside the father node.
	/** \retval the new node of the object.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the object definition with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(ConstXMLNodeRef inFatherNode) = 0;

	/// Return the id
	virtual tUUID					getId() const = 0;

	/// Set the id
	virtual void					setId(tUUID inId) = 0;

	/// Return the localized names of this object definition
	/** For consulting */
	virtual const IAtlString&		getName() const = 0;

	/// Return the localized names of this object definition
	/** For editing */
	virtual IAtlString&				getName() = 0;

	/// Set the name of the definition.
	virtual void					setName(tUtf8 inName) = 0;

	/// Get the class of this object definition.
	virtual tUtf8					getClass() const = 0;

	/// Set the class of this object definition.
	virtual void					setClass(tUtf8 inClassName) = 0;

	/// Get the anchor point
	virtual const CAtlVector&		getAnchorPoint() const = 0;

	/// Get the anchor point
	virtual void					setAnchorPoint(const CAtlVector& inAnchor) = 0;

	/// Get the placement property
	virtual eObjectPlacement		getPlacementProperty() const = 0;

	/// Set the placement property
	virtual void					setPlacementProperty(eObjectPlacement inPlacement) = 0;

	/// Get the up vector
	virtual const CAtlVector&		getUpVector() const = 0;

	/// Get the up vector
	virtual void					setUpVector(const CAtlVector& inUp) = 0;

	/// Get the initial angle
	virtual double					getInitialAngle() const = 0;
	/// Set the initial angle
	virtual void					setInitialAngle(double inAngle) = 0;

	/// Get the rotation effectued to the model
	virtual const CAtlVector&		getModelRotation() const = 0;
	/// Set the rotation effectued to the model
	virtual void					setModelRotation(const CAtlVector& inRotation) = 0;

	/// Get the matrix which corresponds to all rotations effectued to the model.
	/** When you do a rotation of the model, you must use the invert of this matrix to put the model as it was before all rotations. */
	virtual const CAtlMatrix&		getModelRotationMatrix() const = 0;
	/// Set the matrix which corresponds to all rotations effectued to the model
	virtual void					setModelRotationMatrix(const CAtlMatrix& inMatrix) = 0;

	/// Get the position of the model
	virtual const CAtlVector&		getModelPosition() const = 0;
	/// Set the position of the model
	virtual void					setModelPosition(const CAtlVector& inPosition) = 0;

	/// Get the current placement of the origin of the model
	virtual eModelOrigin			getModelOrigin() const = 0;
	/// Set the current placement of the origin of the model
	virtual void					setModelOrigin(eModelOrigin inOrigin) = 0;

	/// Insure that the XML node ref contain up to date values
	virtual void 					synchronizeXML() const = 0;

	/// Get a non modifiable XML data of this definition.
	virtual ConstXMLNodeRef			getXMLNodeRef() const = 0;

	/// Get a modifiable XML data of this definition.
	virtual XMLNodeRef				getXMLNodeRef() = 0;

	/// Get a non modifiable the XML data to be used as default values for instance.
	virtual ConstXMLNodeRef			getInstanceDefault() const = 0;

	/// Get a modifiable XML data to be used as default values for instance.
	virtual XMLNodeRef				getInstanceDefault() = 0;

	/// Get the alternate affectable properties of the AtlObject.
	/** For consulting */
	virtual const AtlPropertyMgr*	getPropertyMgr() const = 0;

	/// Get the alternate affectable properties of the AtlObject.
	/** For setting */
	virtual AtlPropertyMgr*			getPropertyMgr() = 0;

	/// Get the managers of points
	/** For consulting */
	virtual const AtlPointMgr*		getPointMgr() const = 0;

	/// Get the managers of points
	/** For setting */
	virtual AtlPointMgr*			getPointMgr() = 0;

	/// Get the managers of polygons
	/** For consulting */
	virtual const AtlPolygonMgr*	getPolygonMgr() const = 0;

	/// Get the managers of polygons
	/** For setting */
	virtual AtlPolygonMgr*			getPolygonMgr() = 0;

	/// Get Description 1, for consulting
	virtual const IAtlString&		getDescription1() const = 0;
	/// Get Description 1, for editing
	virtual IAtlString&				getDescription1() = 0;

	/// Get Description 2, for consulting
	virtual const IAtlString&		getDescription2() const = 0;
	/// Get Description 2, for editing
	virtual IAtlString&				getDescription2() = 0;

	/// Get additional comments of the definition, for consulting
	virtual const IAtlString&		getAdditionalComments() const = 0;
	/// Get additional comments of the definition, for editing
	virtual IAtlString&				getAdditionalComments() = 0;

	/// Get user interfaces private value
	virtual int						getUIValue() const = 0;

	/// Set user interfaces private value
	virtual void					setUIValue(int inValue) = 0;

	/// Get the light group ID to be exported
	virtual tUUID					getLightGroupID() const = 0;
	/// Set the light group ID to be exported
	virtual void					setLightGroupID(tUUID inID) = 0;

	/// Get the list of all available setups.
	/** For consulting */
	virtual const AtlSetupsMgr*			getSetupsMgr() const = 0;

	/// Get the list of all available setups.
	/** For editing */
	virtual AtlSetupsMgr*			getSetupsMgr() = 0;

	/// Return the atl object.
	/** Return NULL if it's not a 3D plant */
	virtual const IAtl3DPlant* 		as3DPlant() const = 0;

	/// Return the atl object.
	/** Return NULL if it's not a 3D plant */
	virtual IAtl3DPlant* 			as3DPlant() = 0;

	/// Return the atl object
	/** Return NULL if it's not a light object */
	virtual const IAtlObjectLight*	asObjectLight() const = 0;
	/// Return the atl object
	/** Return NULL if it's not a light object */
	virtual IAtlObjectLight*		asObjectLight() = 0;

	/// Return the atl object.
	/** Return NULL if it's not an animated object */
	virtual const IAtlAnimatedObject*		asAnimatedObject() const = 0;

	/// Return the atl object.
	/** Return NULL if it's not an animated object */
	virtual IAtlAnimatedObject*				asAnimatedObject() = 0;
		
	/// Returns the geometry level, from modeler or from Artlantis. See 
	virtual unsigned int getGeometryLevel() const = 0;


	/// Set the geometry level, from modeler or from Artlantis
	virtual  void setGeometryLevel(unsigned int inGeometryLevel) = 0;
	

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~IAtlAtlObjectDefinition() {}
};


} // namespace  AvAtl

#endif
