/*!
	\file	IAtlSequence.h
	\date	04 08 05
	\author	Abvent R&D copyright (c) 2005
	\author	Guillaume Boulet-Gercourt
	Header for the IAtlSequence interface.
*/
#ifndef		IAtlSequence_h
# define	IAtlSequence_h

// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif
#ifndef	CAtlVector_h
	#include "CAtlVector.h"
#endif

namespace	AvAtl {

/// List of sequences.
typedef TAtlObjectMgr<class IAtlSequence>			AtlSequencesMgr;

/*!
	\interface	IAtlSequence
	\date		04 08 05
	\author		Abvent R&D copyright (C) 2005
	\author		Guillaume Boulet-Gercourt
	\brief		Describes a sequence for Artlantis Studio
*/
class	IAtlSequence
{
public:
	/// Put an XML representation of the sequence inside the father node.
	/** \retval the new node of the sequence.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the sequence with the XML.
	/** \retval true intialisation from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// get / set the start frame of the sequence.
	virtual int					getStartFrame() const = 0;
	virtual void				setStartFrame(int inFrameNbr) = 0;

	/// get / set the end frame of the sequence.
	virtual int					getEndFrame() const = 0;
	virtual void				setEndFrame(int inFrameNbr) = 0;

	/// Return the name of the sequence.
	/** Do not delete this string. */
	virtual tUtf8					getName() const = 0;

	/// Set the name of the sequence.
	/** \param inName the new camara's name in unicode. */
	virtual void					setName(tUtf8 inName) = 0;

	/// sequence location in the 3D space.
	virtual const CAtlVector&		getPosition() const = 0;
	/// sequence location in the 3D space.
	/** \param inPosition the point where the sequence is. */
	virtual void					setPosition(const CAtlVector& inPosition) = 0;


	enum	eTargetType
	{
		kFixedVertex = 0,
		kForwardOnPath,
		kRelativeAngle,
		kOnObject
	};

	/// get / set the type of the target
	virtual eTargetType				getTargetType() const = 0;
	virtual void					setTargetType(eTargetType inType) = 0;

	/// A 3D point the sequence looks at.
	virtual const CAtlVector&		getTarget() const = 0;
	/// A 3D point the sequence looks at.
	/** \param inTarget the point that the sequence looks at. */
	virtual void					setTarget(const CAtlVector& inTarget) = 0;

	/// get / set the distance between target and position. Used if the type of target is kForwardOnPath
	virtual double					getTargetDistance() const = 0;
	virtual void					setTargetDistance(double inDistance) = 0;

	/// get / set the horizontal angle of the target. in radians
	virtual double					getTargetAngleH() const = 0;
	virtual void					setTargetAngleH(double inAngle) = 0;

	/// get / set the vertical angle of the target. in radians
	virtual double					getTargetAngleV() const = 0;
	virtual void					setTargetAngleV(double inAngle) = 0;

	/// get / set the current tangent of the path at the position
	virtual const CAtlVector&		getCurrentPathDirection() const = 0;
	virtual void					setCurrentPathDirection(const CAtlVector& inTangent) = 0;

	/// Is this sequence the main view?
	virtual bool					getViewMainFlag() const = 0;
	/// Set this sequence as the main view.
	virtual void					setViewMainFlag() = 0;

	/// Is this sequence rendered by requesting Render All sequences?
	/** \retval true the sequence is rendered by requesting Render All sequences.
		 \retval false the sequence will not be rendered by requesting Render All sequences. */
	virtual bool					getViewOnFlag() const = 0;
	/// Set the state to be rendered by requesting Render All sequences.
	/** \param inState true if the sequence must be rendered by requesting Render All sequences. */
	virtual void					setViewOnFlag(bool inState) = 0;

	/// Return the flags of the view
	virtual int							getFlags() const = 0;
	/// Set the flags of the view
	virtual void						setFlags(int flags) = 0;

	/// Roll defines a sequence's roll angle relative to the horizon.
	/** \return angle in radians [0.0..2pi]. */
	virtual double					getRoll() const = 0;
	/// Roll defines a sequence's roll angle relative to the horizon.
	/** \param inRoll angle in radians [0.0..2pi]. */
	virtual void					setRoll(double inRoll) = 0;

	/// Focal angle. The smaller the angle, the greater the zoom.
	/** Value must be between 1.0 and 180.0. */
	virtual double					getFocal() const = 0;
	/// Focal angle. The smaller the angle, the greater the zoom.
	/** Value must be between 1.0 and 180.0. */
	virtual void					setFocal(double focal) = 0;

	/// Return the view info of this view.
	/** For consulting */
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	/** For setting */
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	virtual tUUID		getId() const = 0;
	virtual void		setId(tUUID inID) = 0;

	/// Return the options of the animation.
	/** For consulting */
	virtual const class IAtlViewOpt*	getViewOpt() const = 0;
	/// Return the options of the animation.
	/** For setting */
	virtual class IAtlViewOpt*			getViewOpt() = 0;

	/// Get the associated object instance uuid (used when the target behavior is set on "kOnObject")
	virtual tUUID						getTargetObjectUUID() const = 0;
	/// Set the associated object instance uuid
	virtual void						setTargetObjectUUID(tUUID inId) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual ~IAtlSequence() {}
};

}

#endif
