/**
 \file ITCENode.h
 
 Interface class for tce node.
 
 \verbatim
 
 File			:	ITCENode.h
 Author			:	Richard Young
 Creation date	:	28/04/2001
 Description	:	The interface IChessNode is the class for tce node.
 
 Modifications	:
 
 \endverbatim
 */


#ifndef ITCENode_h
#define ITCENode_h


#ifndef AtlTypes_h
	#include "CATL/AvAtlTypes.h"
#endif
#ifndef TAtlAutoPtr_h
	#include "CATL/TAtlAutoPtr.h"
#endif


namespace AvAtl{


// Forward declarations.
class IRootNode;
class IChessNode;
class IMappingNode;
class ITurbulenceNode;
class ILeafNode;
class IFresnelNode;
class IFresnelDiffuseNode;
class IGlazingNode;
class ICustomNode;
class IFuture1Node;
class IFuture2Node;
class IFuture3Node;
class IFuture4Node;


// Base class for tce nodes
class ITCENode
{
public:
	// Type of nodes.
	enum ENodeType
	{
		eRootNode,
		eChessNode,
		eMappingNode,
		eTurbulenceNode,
		eLeafNode,
		eFresnelNode,
		eFresnelDiffuseNode,
		eGlazingNode,
		eCustomNode,
		eFuture1Node,
		eFuture2Node,
		eFuture3Node,
		eFuture4Node
	};

	/// Automatic pointer for TCE node.
	typedef TAtlAutoPtr<ITCENode>	AutoPtr;

	/// Increment the number of referents
	virtual void				AddRef() = 0;

	/// Decrement the number of referents
	virtual void				Release() = 0;

	// Return the parent node.
	virtual ITCENode*			getParent() const = 0;

	// This method is used internaly... please dont call this function.
	virtual void				setParent(ITCENode* inParent) = 0;

	/// Return the number of childs.
	virtual int					getNbChilds() const = 0;

	/// Return the specified child.
	virtual const ITCENode*		getChild(int inIndex) const = 0;

	/// Return the specified child.
	virtual ITCENode*			getChild(int inIndex) = 0;

	/// Set the child at the specified index.
	virtual void				setChild(int inIndex, ITCENode* inChild) = 0;

	/// Add this child at the end.
	virtual void				addChild(ITCENode* inChild) = 0;

	/// Remove the specified child.
	virtual void				removeChild(int inIndex) = 0;

	/// Return the id of this node.
	virtual tUUID				getNodeId() const = 0;

	/// Set the id of this node.
	virtual void				setNodeId(tUUID inId) = 0;
	
	/// Return this node or child node that have the specified id.
	/** Return NULL if no child have this Id */
	virtual const ITCENode*		searchNodeId(tUUID inId) const = 0;
	
	/// Return this node or child node that have the specified id.
	/** Return NULL if no child have this Id */
	virtual ITCENode*			searchNodeId(tUUID inId) = 0;
	
	/// Get user interfaces private value
	virtual int					getUIValue() const = 0;

	/// Set user interfaces private value
	virtual void				setUIValue(int inUIValue) = 0;

	/// Return the type of the node.
	virtual ENodeType			getType() const = 0;

	// Smart casters.
	virtual IRootNode*			asRootNode() { return NULL; }
	const IRootNode*			asRootNode() const { return ((ITCENode*)this)->asRootNode(); }
	virtual IChessNode*			asChessNode() { return NULL; }
	const IChessNode*			asChessNode() const { return ((ITCENode*)this)->asChessNode(); }
	virtual IMappingNode*		asMappingNode() { return NULL; }
	const IMappingNode*			asMappingNode() const { return ((ITCENode*)this)->asMappingNode(); }
	virtual ITurbulenceNode*	asTurbulenceNode() { return NULL; }
	const ITurbulenceNode*		asTurbulenceNode() const { return ((ITCENode*)this)->asTurbulenceNode(); }
	virtual ILeafNode*			asLeafNode() { return NULL; }
	const ILeafNode*			asLeafNode() const { return ((ITCENode*)this)->asLeafNode(); }
	virtual IFresnelNode*		asFresnelNode() { return NULL; }
	const IFresnelNode*			asFresnelNode() const { return ((ITCENode*)this)->asFresnelNode(); }
	virtual IFresnelDiffuseNode*asFresnelDiffuseNode() { return NULL; }
	const IFresnelDiffuseNode*	asFresnelDiffuseNode() const { return ((ITCENode*)this)->asFresnelDiffuseNode(); }
	virtual IGlazingNode*		asGlazingNode() { return NULL; }
	const IGlazingNode*			asGlazingNode() const { return ((ITCENode*)this)->asGlazingNode(); }
	virtual ICustomNode*		asCustomNode() { return NULL; }
	const ICustomNode*			asCustomNode() const { return ((ITCENode*)this)->asCustomNode(); }
	virtual IFuture1Node*		asFuture1Node() { return NULL; }
	const IFuture1Node*			asFuture1Node() const { return ((ITCENode*)this)->asFuture1Node(); }
	virtual IFuture2Node*		asFuture2Node() { return NULL; }
	const IFuture2Node*			asFuture2Node() const { return ((ITCENode*)this)->asFuture2Node(); }
	virtual IFuture3Node*		asFuture3Node() { return NULL; }
	const IFuture3Node*			asFuture3Node() const { return ((ITCENode*)this)->asFuture3Node(); }
	virtual IFuture4Node*		asFuture4Node() { return NULL; }
	const IFuture4Node*			asFuture4Node() const { return ((ITCENode*)this)->asFuture4Node(); }

protected:
	// Don't delete, use refcount.
								~ITCENode() {};

	static ITCENode*			clone(ITCENode* parent, const ITCENode* inOther);
};


/// gbg 17 09 2007 : Create a define for limit of values....
#define	ATL_GLOSSY_MIN_VALUE	0.0
#define	ATL_GLOSSY_MAX_VALUE	1000.0

} // namespace  AvAtl


// All the following headers must be include after this one for the build of ATL
#include "CATL/IRootNode.h"
#include "CATL/IChessNode.h"
#include "CATL/IMappingNode.h" // For now, only IMappingNode is public
#include "CATL/ITurbulenceNode.h"
#include "CATL/ILeafNode.h"
#include "CATL/IFresnelNode.h"
#include "CATL/IFresnelDiffuseNode.h"
#include "CATL/IGlazingNode.h"
#include "CATL/ICustomNode.h"


#endif
